import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestGDBServerTargetXML(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("X86")
    def test_x86_64_regs(self):
        """Test grabbing various x86_64 registers from gdbserver."""

        class MyResponder(MockGDBServerResponder):
            reg_data = (
                (
                    "0102030405060708"  # rcx
                    "1112131415161718"  # rdx
                    "2122232425262728"  # rsi
                    "3132333435363738"  # rdi
                    "4142434445464748"  # rbp
                    "5152535455565758"  # rsp
                    "6162636465666768"  # r8
                    "7172737475767778"  # r9
                    "8182838485868788"  # rip
                    "91929394"  # eflags
                    "0102030405060708090a"  # st0
                    "1112131415161718191a"  # st1
                )
                + 6 * ("2122232425262728292a")  # st2..st7
                + (
                    "8182838485868788898a8b8c8d8e8f90"  # xmm0
                    "9192939495969798999a9b9c9d9e9fa0"  # xmm1
                )
                + 14 * ("a1a2a3a4a5a6a7a8a9aaabacadaeafb0")  # xmm2..xmm15
                + ("00000000")  # mxcsr
                + (
                    "b1b2b3b4b5b6b7b8b9babbbcbdbebfc0"  # ymm0h
                    "c1c2c3c4c5c6c7c8c9cacbcccdcecfd0"  # ymm1h
                )
                + 14 * ("d1d2d3d4d5d6d7d8d9dadbdcdddedfe0")  # ymm2h..ymm15h
            )

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                        <target>
                          <architecture>i386:x86-64</architecture>
                          <osabi>GNU/Linux</osabi>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="rcx" bitsize="64" type="int64" regnum="2"/>
                            <reg name="rdx" bitsize="64" type="int64" regnum="3"/>
                            <reg name="rsi" bitsize="64" type="int64" regnum="4"/>
                            <reg name="rdi" bitsize="64" type="int64" regnum="5"/>
                            <reg name="rbp" bitsize="64" type="data_ptr" regnum="6"/>
                            <reg name="rsp" bitsize="64" type="data_ptr" regnum="7"/>
                            <reg name="r8" bitsize="64" type="int64" regnum="8"/>
                            <reg name="r9" bitsize="64" type="int64" regnum="9"/>
                            <reg name="rip" bitsize="64" type="code_ptr" regnum="16"/>
                            <reg name="eflags" bitsize="32" type="i386_eflags" regnum="17"/>
                            <reg name="st0" bitsize="80" type="i387_ext" regnum="24"/>
                            <reg name="st1" bitsize="80" type="i387_ext" regnum="25"/>
                            <reg name="st2" bitsize="80" type="i387_ext" regnum="26"/>
                            <reg name="st3" bitsize="80" type="i387_ext" regnum="27"/>
                            <reg name="st4" bitsize="80" type="i387_ext" regnum="28"/>
                            <reg name="st5" bitsize="80" type="i387_ext" regnum="29"/>
                            <reg name="st6" bitsize="80" type="i387_ext" regnum="30"/>
                            <reg name="st7" bitsize="80" type="i387_ext" regnum="31"/>
                          </feature>
                          <feature name="org.gnu.gdb.i386.sse">
                            <reg name="xmm0" bitsize="128" type="vec128" regnum="40"/>
                            <reg name="xmm1" bitsize="128" type="vec128" regnum="41"/>
                            <reg name="xmm2" bitsize="128" type="vec128" regnum="42"/>
                            <reg name="xmm3" bitsize="128" type="vec128" regnum="43"/>
                            <reg name="xmm4" bitsize="128" type="vec128" regnum="44"/>
                            <reg name="xmm5" bitsize="128" type="vec128" regnum="45"/>
                            <reg name="xmm6" bitsize="128" type="vec128" regnum="46"/>
                            <reg name="xmm7" bitsize="128" type="vec128" regnum="47"/>
                            <reg name="xmm8" bitsize="128" type="vec128" regnum="48"/>
                            <reg name="xmm9" bitsize="128" type="vec128" regnum="49"/>
                            <reg name="xmm10" bitsize="128" type="vec128" regnum="50"/>
                            <reg name="xmm11" bitsize="128" type="vec128" regnum="51"/>
                            <reg name="xmm12" bitsize="128" type="vec128" regnum="52"/>
                            <reg name="xmm13" bitsize="128" type="vec128" regnum="53"/>
                            <reg name="xmm14" bitsize="128" type="vec128" regnum="54"/>
                            <reg name="xmm15" bitsize="128" type="vec128" regnum="55"/>
                            <reg name="mxcsr" bitsize="32" type="i386_mxcsr" regnum="56" group="vector"/>
                          </feature>
                          <feature name="org.gnu.gdb.i386.avx">
                            <reg name="ymm0h" bitsize="128" type="uint128" regnum="60"/>
                            <reg name="ymm1h" bitsize="128" type="uint128" regnum="61"/>
                            <reg name="ymm2h" bitsize="128" type="uint128" regnum="62"/>
                            <reg name="ymm3h" bitsize="128" type="uint128" regnum="63"/>
                            <reg name="ymm4h" bitsize="128" type="uint128" regnum="64"/>
                            <reg name="ymm5h" bitsize="128" type="uint128" regnum="65"/>
                            <reg name="ymm6h" bitsize="128" type="uint128" regnum="66"/>
                            <reg name="ymm7h" bitsize="128" type="uint128" regnum="67"/>
                            <reg name="ymm8h" bitsize="128" type="uint128" regnum="68"/>
                            <reg name="ymm9h" bitsize="128" type="uint128" regnum="69"/>
                            <reg name="ymm10h" bitsize="128" type="uint128" regnum="70"/>
                            <reg name="ymm11h" bitsize="128" type="uint128" regnum="71"/>
                            <reg name="ymm12h" bitsize="128" type="uint128" regnum="72"/>
                            <reg name="ymm13h" bitsize="128" type="uint128" regnum="73"/>
                            <reg name="ymm14h" bitsize="128" type="uint128" regnum="74"/>
                            <reg name="ymm15h" bitsize="128" type="uint128" regnum="75"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return self.reg_data

            def writeRegisters(self, reg_hex):
                self.reg_data = reg_hex
                return "OK"

            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;07:0102030405060708;10:1112131415161718;"

        self.server.responder = MyResponder()

        target = self.createTarget("basic_eh_frame.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        # test generic aliases
        self.match("register read arg4", ["rcx = 0x0807060504030201"])
        self.match("register read arg3", ["rdx = 0x1817161514131211"])
        self.match("register read arg2", ["rsi = 0x2827262524232221"])
        self.match("register read arg1", ["rdi = 0x3837363534333231"])
        self.match("register read fp", ["rbp = 0x4847464544434241"])
        self.match("register read sp", ["rsp = 0x5857565554535251"])
        self.match("register read arg5", ["r8 = 0x6867666564636261"])
        self.match("register read arg6", ["r9 = 0x7877767574737271"])
        self.match("register read pc", ["rip = 0x8887868584838281"])
        self.match("register read flags", ["eflags = 0x94939291"])

        # both stX and xmmX should be displayed as vectors
        self.match(
            "register read st0",
            ["st0 = {0x01 0x02 0x03 0x04 0x05 0x06 0x07 0x08 0x09 0x0a}"],
        )
        self.match(
            "register read st1",
            ["st1 = {0x11 0x12 0x13 0x14 0x15 0x16 0x17 0x18 0x19 0x1a}"],
        )
        self.match(
            "register read xmm0",
            [
                "xmm0 = {0x81 0x82 0x83 0x84 0x85 0x86 0x87 0x88 "
                "0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 0x90}"
            ],
        )
        self.match(
            "register read xmm1",
            [
                "xmm1 = {0x91 0x92 0x93 0x94 0x95 0x96 0x97 0x98 "
                "0x99 0x9a 0x9b 0x9c 0x9d 0x9e 0x9f 0xa0}"
            ],
        )

        # test pseudo-registers
        self.filecheck(
            "register read --all",
            os.path.join(os.path.dirname(__file__), "amd64-partial-regs.FileCheck"),
        )

        # test writing into pseudo-registers
        self.runCmd("register write ecx 0xfffefdfc")
        self.match("register read rcx", ["rcx = 0x08070605fffefdfc"])

        self.runCmd("register write cx 0xfbfa")
        self.match("register read ecx", ["ecx = 0xfffefbfa"])
        self.match("register read rcx", ["rcx = 0x08070605fffefbfa"])

        self.runCmd("register write ch 0xf9")
        self.match("register read cx", ["cx = 0xf9fa"])
        self.match("register read ecx", ["ecx = 0xfffef9fa"])
        self.match("register read rcx", ["rcx = 0x08070605fffef9fa"])

        self.runCmd("register write cl 0xf8")
        self.match("register read cx", ["cx = 0xf9f8"])
        self.match("register read ecx", ["ecx = 0xfffef9f8"])
        self.match("register read rcx", ["rcx = 0x08070605fffef9f8"])

        self.runCmd("register write mm0 0xfffefdfcfbfaf9f8")
        self.match(
            "register read st0",
            ["st0 = {0xf8 0xf9 0xfa 0xfb 0xfc 0xfd 0xfe 0xff 0x09 0x0a}"],
        )

        self.runCmd(
            'register write xmm0 "{0xff 0xfe 0xfd 0xfc 0xfb 0xfa 0xf9 '
            '0xf8 0xf7 0xf6 0xf5 0xf4 0xf3 0xf2 0xf1 0xf0}"'
        )
        self.match(
            "register read ymm0",
            [
                "ymm0 = {0xff 0xfe 0xfd 0xfc 0xfb 0xfa 0xf9 0xf8 0xf7 0xf6 "
                "0xf5 0xf4 0xf3 0xf2 0xf1 0xf0 0xb1 0xb2 0xb3 0xb4 0xb5 "
                "0xb6 0xb7 0xb8 0xb9 0xba 0xbb 0xbc 0xbd 0xbe 0xbf 0xc0}"
            ],
        )

        self.runCmd(
            'register write ymm0h "{0xef 0xee 0xed 0xec 0xeb 0xea 0xe9 '
            '0xe8 0xe7 0xe6 0xe5 0xe4 0xe3 0xe2 0xe1 0xe0}"'
        )
        self.match(
            "register read ymm0",
            [
                "ymm0 = {0xff 0xfe 0xfd 0xfc 0xfb 0xfa 0xf9 0xf8 0xf7 0xf6 "
                "0xf5 0xf4 0xf3 0xf2 0xf1 0xf0 0xef 0xee 0xed 0xec 0xeb "
                "0xea 0xe9 0xe8 0xe7 0xe6 0xe5 0xe4 0xe3 0xe2 0xe1 0xe0}"
            ],
        )

        self.runCmd(
            'register write ymm0 "{0xd0 0xd1 0xd2 0xd3 0xd4 0xd5 0xd6 '
            "0xd7 0xd8 0xd9 0xda 0xdb 0xdc 0xdd 0xde 0xdf 0xe0 0xe1 "
            "0xe2 0xe3 0xe4 0xe5 0xe6 0xe7 0xe8 0xe9 0xea 0xeb 0xec "
            '0xed 0xee 0xef}"'
        )
        self.match(
            "register read ymm0",
            [
                "ymm0 = {0xd0 0xd1 0xd2 0xd3 0xd4 0xd5 0xd6 0xd7 0xd8 0xd9 "
                "0xda 0xdb 0xdc 0xdd 0xde 0xdf 0xe0 0xe1 0xe2 0xe3 0xe4 "
                "0xe5 0xe6 0xe7 0xe8 0xe9 0xea 0xeb 0xec 0xed 0xee 0xef}"
            ],
        )

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("X86")
    def test_i386_regs(self):
        """Test grabbing various i386 registers from gdbserver."""

        class MyResponder(MockGDBServerResponder):
            reg_data = (
                (
                    "01020304"  # eax
                    "11121314"  # ecx
                    "21222324"  # edx
                    "31323334"  # ebx
                    "41424344"  # esp
                    "51525354"  # ebp
                    "61626364"  # esi
                    "71727374"  # edi
                    "81828384"  # eip
                    "91929394"  # eflags
                    "0102030405060708090a"  # st0
                    "1112131415161718191a"  # st1
                )
                + 6 * ("2122232425262728292a")  # st2..st7
                + (
                    "8182838485868788898a8b8c8d8e8f90"  # xmm0
                    "9192939495969798999a9b9c9d9e9fa0"  # xmm1
                )
                + 6 * ("a1a2a3a4a5a6a7a8a9aaabacadaeafb0")  # xmm2..xmm7
                + ("00000000")  # mxcsr
                + (
                    "b1b2b3b4b5b6b7b8b9babbbcbdbebfc0"  # ymm0h
                    "c1c2c3c4c5c6c7c8c9cacbcccdcecfd0"  # ymm1h
                )
                + 6 * ("d1d2d3d4d5d6d7d8d9dadbdcdddedfe0")  # ymm2h..ymm7h
            )

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                        <target>
                          <architecture>i386</architecture>
                          <osabi>GNU/Linux</osabi>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="eax" bitsize="32" type="int32" regnum="0"/>
                            <reg name="ecx" bitsize="32" type="int32" regnum="1"/>
                            <reg name="edx" bitsize="32" type="int32" regnum="2"/>
                            <reg name="ebx" bitsize="32" type="int32" regnum="3"/>
                            <reg name="esp" bitsize="32" type="data_ptr" regnum="4"/>
                            <reg name="ebp" bitsize="32" type="data_ptr" regnum="5"/>
                            <reg name="esi" bitsize="32" type="int32" regnum="6"/>
                            <reg name="edi" bitsize="32" type="int32" regnum="7"/>
                            <reg name="eip" bitsize="32" type="code_ptr" regnum="8"/>
                            <reg name="eflags" bitsize="32" type="i386_eflags" regnum="9"/>
                            <reg name="st0" bitsize="80" type="i387_ext" regnum="16"/>
                            <reg name="st1" bitsize="80" type="i387_ext" regnum="17"/>
                            <reg name="st2" bitsize="80" type="i387_ext" regnum="18"/>
                            <reg name="st3" bitsize="80" type="i387_ext" regnum="19"/>
                            <reg name="st4" bitsize="80" type="i387_ext" regnum="20"/>
                            <reg name="st5" bitsize="80" type="i387_ext" regnum="21"/>
                            <reg name="st6" bitsize="80" type="i387_ext" regnum="22"/>
                            <reg name="st7" bitsize="80" type="i387_ext" regnum="23"/>
                          </feature>
                          <feature name="org.gnu.gdb.i386.sse">
                            <reg name="xmm0" bitsize="128" type="vec128" regnum="32"/>
                            <reg name="xmm1" bitsize="128" type="vec128" regnum="33"/>
                            <reg name="xmm2" bitsize="128" type="vec128" regnum="34"/>
                            <reg name="xmm3" bitsize="128" type="vec128" regnum="35"/>
                            <reg name="xmm4" bitsize="128" type="vec128" regnum="36"/>
                            <reg name="xmm5" bitsize="128" type="vec128" regnum="37"/>
                            <reg name="xmm6" bitsize="128" type="vec128" regnum="38"/>
                            <reg name="xmm7" bitsize="128" type="vec128" regnum="39"/>
                            <reg name="mxcsr" bitsize="32" type="i386_mxcsr" regnum="40" group="vector"/>
                          </feature>
                          <feature name="org.gnu.gdb.i386.avx">
                            <reg name="ymm0h" bitsize="128" type="uint128" regnum="42"/>
                            <reg name="ymm1h" bitsize="128" type="uint128" regnum="43"/>
                            <reg name="ymm2h" bitsize="128" type="uint128" regnum="44"/>
                            <reg name="ymm3h" bitsize="128" type="uint128" regnum="45"/>
                            <reg name="ymm4h" bitsize="128" type="uint128" regnum="46"/>
                            <reg name="ymm5h" bitsize="128" type="uint128" regnum="47"/>
                            <reg name="ymm6h" bitsize="128" type="uint128" regnum="48"/>
                            <reg name="ymm7h" bitsize="128" type="uint128" regnum="49"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return self.reg_data

            def writeRegisters(self, reg_hex):
                self.reg_data = reg_hex
                return "OK"

            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;07:0102030405060708;10:1112131415161718;"

        self.server.responder = MyResponder()

        target = self.createTarget("basic_eh_frame-i386.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        # test generic aliases
        self.match("register read fp", ["ebp = 0x54535251"])
        self.match("register read sp", ["esp = 0x44434241"])
        self.match("register read pc", ["eip = 0x84838281"])
        self.match("register read flags", ["eflags = 0x94939291"])

        # test pseudo-registers
        self.match("register read cx", ["cx = 0x1211"])
        self.match("register read ch", ["ch = 0x12"])
        self.match("register read cl", ["cl = 0x11"])
        self.match("register read mm0", ["mm0 = 0x0807060504030201"])
        self.match("register read mm1", ["mm1 = 0x1817161514131211"])

        # both stX and xmmX should be displayed as vectors
        self.match(
            "register read st0",
            ["st0 = {0x01 0x02 0x03 0x04 0x05 0x06 0x07 0x08 0x09 0x0a}"],
        )
        self.match(
            "register read st1",
            ["st1 = {0x11 0x12 0x13 0x14 0x15 0x16 0x17 0x18 0x19 0x1a}"],
        )
        self.match(
            "register read xmm0",
            [
                "xmm0 = {0x81 0x82 0x83 0x84 0x85 0x86 0x87 0x88 "
                "0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 0x90}"
            ],
        )
        self.match(
            "register read xmm1",
            [
                "xmm1 = {0x91 0x92 0x93 0x94 0x95 0x96 0x97 0x98 "
                "0x99 0x9a 0x9b 0x9c 0x9d 0x9e 0x9f 0xa0}"
            ],
        )

        # test writing into pseudo-registers
        self.runCmd("register write cx 0xfbfa")
        self.match("register read ecx", ["ecx = 0x1413fbfa"])

        self.runCmd("register write ch 0xf9")
        self.match("register read cx", ["cx = 0xf9fa"])
        self.match("register read ecx", ["ecx = 0x1413f9fa"])

        self.runCmd("register write cl 0xf8")
        self.match("register read cx", ["cx = 0xf9f8"])
        self.match("register read ecx", ["ecx = 0x1413f9f8"])

        self.runCmd("register write mm0 0xfffefdfcfbfaf9f8")
        self.match(
            "register read st0",
            ["st0 = {0xf8 0xf9 0xfa 0xfb 0xfc 0xfd 0xfe 0xff 0x09 0x0a}"],
        )

        self.runCmd(
            'register write xmm0 "{0xff 0xfe 0xfd 0xfc 0xfb 0xfa 0xf9 '
            '0xf8 0xf7 0xf6 0xf5 0xf4 0xf3 0xf2 0xf1 0xf0}"'
        )
        self.match(
            "register read ymm0",
            [
                "ymm0 = {0xff 0xfe 0xfd 0xfc 0xfb 0xfa 0xf9 0xf8 0xf7 0xf6 "
                "0xf5 0xf4 0xf3 0xf2 0xf1 0xf0 0xb1 0xb2 0xb3 0xb4 0xb5 "
                "0xb6 0xb7 0xb8 0xb9 0xba 0xbb 0xbc 0xbd 0xbe 0xbf 0xc0}"
            ],
        )

        self.runCmd(
            'register write ymm0h "{0xef 0xee 0xed 0xec 0xeb 0xea 0xe9 '
            '0xe8 0xe7 0xe6 0xe5 0xe4 0xe3 0xe2 0xe1 0xe0}"'
        )
        self.match(
            "register read ymm0",
            [
                "ymm0 = {0xff 0xfe 0xfd 0xfc 0xfb 0xfa 0xf9 0xf8 0xf7 0xf6 "
                "0xf5 0xf4 0xf3 0xf2 0xf1 0xf0 0xef 0xee 0xed 0xec 0xeb "
                "0xea 0xe9 0xe8 0xe7 0xe6 0xe5 0xe4 0xe3 0xe2 0xe1 0xe0}"
            ],
        )

        self.runCmd(
            'register write ymm0 "{0xd0 0xd1 0xd2 0xd3 0xd4 0xd5 0xd6 '
            "0xd7 0xd8 0xd9 0xda 0xdb 0xdc 0xdd 0xde 0xdf 0xe0 0xe1 "
            "0xe2 0xe3 0xe4 0xe5 0xe6 0xe7 0xe8 0xe9 0xea 0xeb 0xec "
            '0xed 0xee 0xef}"'
        )
        self.match(
            "register read ymm0",
            [
                "ymm0 = {0xd0 0xd1 0xd2 0xd3 0xd4 0xd5 0xd6 0xd7 0xd8 0xd9 "
                "0xda 0xdb 0xdc 0xdd 0xde 0xdf 0xe0 0xe1 0xe2 0xe3 0xe4 "
                "0xe5 0xe6 0xe7 0xe8 0xe9 0xea 0xeb 0xec 0xed 0xee 0xef}"
            ],
        )

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("AArch64")
    def test_aarch64_regs(self):
        """Test grabbing various aarch64 registers from gdbserver."""

        class MyResponder(MockGDBServerResponder):
            reg_data = (
                ("0102030405060708" "1112131415161718")  # x0  # x1
                + 27 * ("2122232425262728")  # x2..x28
                + (
                    "3132333435363738"  # x29 (fp)
                    "4142434445464748"  # x30 (lr)
                    "5152535455565758"  # x31 (sp)
                    "6162636465666768"  # pc
                    "71727374"  # cpsr
                    "8182838485868788898a8b8c8d8e8f90"  # v0
                    "9192939495969798999a9b9c9d9e9fa0"  # v1
                )
                + 30 * ("a1a2a3a4a5a6a7a8a9aaabacadaeafb0")  # v2..v31
                + ("00000000" "00000000")  # fpsr  # fpcr
            )

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                        <target>
                          <architecture>aarch64</architecture>
                          <feature name="org.gnu.gdb.aarch64.core">
                            <reg name="x0" bitsize="64" type="int" regnum="0"/>
                            <reg name="x1" bitsize="64" type="int" regnum="1"/>
                            <reg name="x2" bitsize="64" type="int" regnum="2"/>
                            <reg name="x3" bitsize="64" type="int" regnum="3"/>
                            <reg name="x4" bitsize="64" type="int" regnum="4"/>
                            <reg name="x5" bitsize="64" type="int" regnum="5"/>
                            <reg name="x6" bitsize="64" type="int" regnum="6"/>
                            <reg name="x7" bitsize="64" type="int" regnum="7"/>
                            <reg name="x8" bitsize="64" type="int" regnum="8"/>
                            <reg name="x9" bitsize="64" type="int" regnum="9"/>
                            <reg name="x10" bitsize="64" type="int" regnum="10"/>
                            <reg name="x11" bitsize="64" type="int" regnum="11"/>
                            <reg name="x12" bitsize="64" type="int" regnum="12"/>
                            <reg name="x13" bitsize="64" type="int" regnum="13"/>
                            <reg name="x14" bitsize="64" type="int" regnum="14"/>
                            <reg name="x15" bitsize="64" type="int" regnum="15"/>
                            <reg name="x16" bitsize="64" type="int" regnum="16"/>
                            <reg name="x17" bitsize="64" type="int" regnum="17"/>
                            <reg name="x18" bitsize="64" type="int" regnum="18"/>
                            <reg name="x19" bitsize="64" type="int" regnum="19"/>
                            <reg name="x20" bitsize="64" type="int" regnum="20"/>
                            <reg name="x21" bitsize="64" type="int" regnum="21"/>
                            <reg name="x22" bitsize="64" type="int" regnum="22"/>
                            <reg name="x23" bitsize="64" type="int" regnum="23"/>
                            <reg name="x24" bitsize="64" type="int" regnum="24"/>
                            <reg name="x25" bitsize="64" type="int" regnum="25"/>
                            <reg name="x26" bitsize="64" type="int" regnum="26"/>
                            <reg name="x27" bitsize="64" type="int" regnum="27"/>
                            <reg name="x28" bitsize="64" type="int" regnum="28"/>
                            <reg name="x29" bitsize="64" type="int" regnum="29"/>
                            <reg name="x30" bitsize="64" type="int" regnum="30"/>
                            <reg name="sp" bitsize="64" type="data_ptr" regnum="31"/>
                            <reg name="pc" bitsize="64" type="code_ptr" regnum="32"/>
                            <reg name="cpsr" bitsize="32" type="cpsr_flags" regnum="33"/>
                          </feature>
                          <feature name="org.gnu.gdb.aarch64.fpu">
                            <reg name="v0" bitsize="128" type="aarch64v" regnum="34"/>
                            <reg name="v1" bitsize="128" type="aarch64v" regnum="35"/>
                            <reg name="v2" bitsize="128" type="aarch64v" regnum="36"/>
                            <reg name="v3" bitsize="128" type="aarch64v" regnum="37"/>
                            <reg name="v4" bitsize="128" type="aarch64v" regnum="38"/>
                            <reg name="v5" bitsize="128" type="aarch64v" regnum="39"/>
                            <reg name="v6" bitsize="128" type="aarch64v" regnum="40"/>
                            <reg name="v7" bitsize="128" type="aarch64v" regnum="41"/>
                            <reg name="v8" bitsize="128" type="aarch64v" regnum="42"/>
                            <reg name="v9" bitsize="128" type="aarch64v" regnum="43"/>
                            <reg name="v10" bitsize="128" type="aarch64v" regnum="44"/>
                            <reg name="v11" bitsize="128" type="aarch64v" regnum="45"/>
                            <reg name="v12" bitsize="128" type="aarch64v" regnum="46"/>
                            <reg name="v13" bitsize="128" type="aarch64v" regnum="47"/>
                            <reg name="v14" bitsize="128" type="aarch64v" regnum="48"/>
                            <reg name="v15" bitsize="128" type="aarch64v" regnum="49"/>
                            <reg name="v16" bitsize="128" type="aarch64v" regnum="50"/>
                            <reg name="v17" bitsize="128" type="aarch64v" regnum="51"/>
                            <reg name="v18" bitsize="128" type="aarch64v" regnum="52"/>
                            <reg name="v19" bitsize="128" type="aarch64v" regnum="53"/>
                            <reg name="v20" bitsize="128" type="aarch64v" regnum="54"/>
                            <reg name="v21" bitsize="128" type="aarch64v" regnum="55"/>
                            <reg name="v22" bitsize="128" type="aarch64v" regnum="56"/>
                            <reg name="v23" bitsize="128" type="aarch64v" regnum="57"/>
                            <reg name="v24" bitsize="128" type="aarch64v" regnum="58"/>
                            <reg name="v25" bitsize="128" type="aarch64v" regnum="59"/>
                            <reg name="v26" bitsize="128" type="aarch64v" regnum="60"/>
                            <reg name="v27" bitsize="128" type="aarch64v" regnum="61"/>
                            <reg name="v28" bitsize="128" type="aarch64v" regnum="62"/>
                            <reg name="v29" bitsize="128" type="aarch64v" regnum="63"/>
                            <reg name="v30" bitsize="128" type="aarch64v" regnum="64"/>
                            <reg name="v31" bitsize="128" type="aarch64v" regnum="65"/>
                            <reg name="fpsr" bitsize="32" type="int" regnum="66"/>
                            <reg name="fpcr" bitsize="32" type="int" regnum="67"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return self.reg_data

            def writeRegisters(self, reg_hex):
                self.reg_data = reg_hex
                return "OK"

            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;07:0102030405060708;10:1112131415161718;"

        self.server.responder = MyResponder()

        target = self.createTarget("basic_eh_frame-aarch64.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        # test GPRs
        self.match("register read x0", ["x0 = 0x0807060504030201"])
        self.match("register read x1", ["x1 = 0x1817161514131211"])
        self.match("register read x29", ["x29 = 0x3837363534333231"])
        self.match("register read x30", ["x30 = 0x4847464544434241"])
        self.match("register read x31", ["sp = 0x5857565554535251"])
        self.match("register read sp", ["sp = 0x5857565554535251"])
        self.match("register read pc", ["pc = 0x6867666564636261"])
        self.match("register read cpsr", ["cpsr = 0x74737271"])

        # test generic aliases
        self.match("register read arg1", ["x0 = 0x0807060504030201"])
        self.match("register read arg2", ["x1 = 0x1817161514131211"])
        self.match("register read fp", ["x29 = 0x3837363534333231"])
        self.match("register read lr", ["x30 = 0x4847464544434241"])
        self.match("register read ra", ["x30 = 0x4847464544434241"])
        self.match("register read flags", ["cpsr = 0x74737271"])

        # test vector registers
        self.match(
            "register read v0",
            [
                "v0 = {0x81 0x82 0x83 0x84 0x85 0x86 0x87 0x88 0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 0x90}"
            ],
        )
        self.match(
            "register read v31",
            [
                "v31 = {0xa1 0xa2 0xa3 0xa4 0xa5 0xa6 0xa7 0xa8 0xa9 0xaa 0xab 0xac 0xad 0xae 0xaf 0xb0}"
            ],
        )

        # test partial registers
        self.match("register read w0", ["w0 = 0x04030201"])
        self.runCmd("register write w0 0xfffefdfc")
        self.match("register read x0", ["x0 = 0x08070605fffefdfc"])

        self.match("register read w1", ["w1 = 0x14131211"])
        self.runCmd("register write w1 0xefeeedec")
        self.match("register read x1", ["x1 = 0x18171615efeeedec"])

        self.match("register read w30", ["w30 = 0x44434241"])
        self.runCmd("register write w30 0xdfdedddc")
        self.match("register read x30", ["x30 = 0x48474645dfdedddc"])

        self.match("register read w31", ["w31 = 0x54535251"])
        self.runCmd("register write w31 0xcfcecdcc")
        self.match("register read x31", ["sp = 0x58575655cfcecdcc"])

        # test FPU registers (overlapping with vector registers)
        self.runCmd("register write d0 16")
        self.match(
            "register read v0",
            [
                "v0 = {0x00 0x00 0x00 0x00 0x00 0x00 0x30 0x40 0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 0x90}"
            ],
        )
        self.runCmd(
            "register write v31 '{0x00 0x00 0x00 0x00 0x00 0x00 0x50 0x40 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff}'"
        )
        self.match("register read d31", ["d31 = 64"])

        self.runCmd("register write s0 32")
        self.match(
            "register read v0",
            [
                "v0 = {0x00 0x00 0x00 0x42 0x00 0x00 0x30 0x40 0x89 0x8a 0x8b 0x8c 0x8d 0x8e 0x8f 0x90}"
            ],
        )
        self.runCmd(
            "register write v31 '{0x00 0x00 0x00 0x43 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff}'"
        )
        self.match("register read s31", ["s31 = 128"])

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("RISCV")
    def test_riscv64_regs(self):
        """Test grabbing various riscv64 registers from gdbserver."""

        class MyResponder(MockGDBServerResponder):
            reg_data = (
                "0102030405060708"  # zero
                "0102030405060708"  # ra
                "0102030405060708"  # sp
                "0102030405060708"  # gp
                "0102030405060708"  # tp
                "0102030405060708"  # t0
                "0102030405060708"  # t1
                "0102030405060708"  # t2
                "0102030405060708"  # fp
                "0102030405060708"  # s1
                "0102030405060708"  # a0
                "0102030405060708"  # a1
                "0102030405060708"  # a2
                "0102030405060708"  # a3
                "0102030405060708"  # a4
                "0102030405060708"  # a5
                "0102030405060708"  # a6
                "0102030405060708"  # a7
                "0102030405060708"  # s2
                "0102030405060708"  # s3
                "0102030405060708"  # s4
                "0102030405060708"  # s5
                "0102030405060708"  # s6
                "0102030405060708"  # s7
                "0102030405060708"  # s8
                "0102030405060708"  # s9
                "0102030405060708"  # s10
                "0102030405060708"  # s11
                "0102030405060708"  # t3
                "0102030405060708"  # t4
                "0102030405060708"  # t5
                "0102030405060708"  # t6
                "6162636465666768"  # pc
                "0000C03F"  # ft0
                "e07a6147a8a40940"  # ft1
            )

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    # Note that this XML does not include any aliases, LLDB must generate them itself.
                    return (
                        """<?xml version="1.0"?>
                        <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                        <target>
                            <architecture>riscv</architecture>
                            <feature name="org.gnu.gdb.riscv.cpu">
                                <reg name="zero" bitsize="64" type="int"/>
                                <reg name="ra" bitsize="64" type="code_ptr"/>
                                <reg name="sp" bitsize="64" type="data_ptr"/>
                                <reg name="gp" bitsize="64" type="data_ptr"/>
                                <reg name="tp" bitsize="64" type="data_ptr"/>
                                <reg name="t0" bitsize="64" type="int"/>
                                <reg name="t1" bitsize="64" type="int"/>
                                <reg name="t2" bitsize="64" type="int"/>
                                <reg name="fp" bitsize="64" type="data_ptr"/>
                                <reg name="s1" bitsize="64" type="int"/>
                                <reg name="a0" bitsize="64" type="int"/>
                                <reg name="a1" bitsize="64" type="int"/>
                                <reg name="a2" bitsize="64" type="int"/>
                                <reg name="a3" bitsize="64" type="int"/>
                                <reg name="a4" bitsize="64" type="int"/>
                                <reg name="a5" bitsize="64" type="int"/>
                                <reg name="a6" bitsize="64" type="int"/>
                                <reg name="a7" bitsize="64" type="int"/>
                                <reg name="s2" bitsize="64" type="int"/>
                                <reg name="s3" bitsize="64" type="int"/>
                                <reg name="s4" bitsize="64" type="int"/>
                                <reg name="s5" bitsize="64" type="int"/>
                                <reg name="s6" bitsize="64" type="int"/>
                                <reg name="s7" bitsize="64" type="int"/>
                                <reg name="s8" bitsize="64" type="int"/>
                                <reg name="s9" bitsize="64" type="int"/>
                                <reg name="s10" bitsize="64" type="int"/>
                                <reg name="s11" bitsize="64" type="int"/>
                                <reg name="t3" bitsize="64" type="int"/>
                                <reg name="t4" bitsize="64" type="int"/>
                                <reg name="t5" bitsize="64" type="int"/>
                                <reg name="t6" bitsize="64" type="int"/>
                                <reg name="pc" bitsize="64" type="code_ptr"/>
                            </feature>
                            <feature name='org.gnu.gdb.riscv.fpu'>
                                <reg name='ft0' bitsize='32' type='ieee_single'/>
                                <reg name='ft1' bitsize='64' type='ieee_double'/>
                            </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return self.reg_data

            def writeRegisters(self, reg_hex):
                self.reg_data = reg_hex
                return "OK"

            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;07:0102030405060708;10:1112131415161718;"

        self.server.responder = MyResponder()

        target = self.createTarget("basic_eh_frame-riscv64.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        # test generic aliases
        self.match("register read x0", ["zero = 0x0807060504030201"])
        self.match("register read x1", ["ra = 0x0807060504030201"])
        self.match("register read x2", ["sp = 0x0807060504030201"])
        self.match("register read x3", ["gp = 0x0807060504030201"])
        self.match("register read x4", ["tp = 0x0807060504030201"])
        self.match("register read x5", ["t0 = 0x0807060504030201"])
        self.match("register read x6", ["t1 = 0x0807060504030201"])
        self.match("register read x7", ["t2 = 0x0807060504030201"])
        # Register x8 is probably not working because it has two aliases fp, s0
        # See issue #127900
        # self.match("register read x8", ["fp = 0x0807060504030201"])
        self.match("register read x9", ["s1 = 0x0807060504030201"])
        self.match("register read x10", ["a0 = 0x0807060504030201"])
        self.match("register read x11", ["a1 = 0x0807060504030201"])
        self.match("register read x12", ["a2 = 0x0807060504030201"])
        self.match("register read x13", ["a3 = 0x0807060504030201"])
        self.match("register read x14", ["a4 = 0x0807060504030201"])
        self.match("register read x15", ["a5 = 0x0807060504030201"])
        self.match("register read x16", ["a6 = 0x0807060504030201"])
        self.match("register read x17", ["a7 = 0x0807060504030201"])
        self.match("register read x18", ["s2 = 0x0807060504030201"])
        self.match("register read x19", ["s3 = 0x0807060504030201"])
        self.match("register read x20", ["s4 = 0x0807060504030201"])
        self.match("register read x21", ["s5 = 0x0807060504030201"])
        self.match("register read x22", ["s6 = 0x0807060504030201"])
        self.match("register read x23", ["s7 = 0x0807060504030201"])
        self.match("register read x24", ["s8 = 0x0807060504030201"])
        self.match("register read x25", ["s9 = 0x0807060504030201"])
        self.match("register read x26", ["s10 = 0x0807060504030201"])
        self.match("register read x27", ["s11 = 0x0807060504030201"])
        self.match("register read x28", ["t3 = 0x0807060504030201"])
        self.match("register read x29", ["t4 = 0x0807060504030201"])
        self.match("register read x30", ["t5 = 0x0807060504030201"])
        self.match("register read x31", ["t6 = 0x0807060504030201"])
        self.match("register read pc", ["pc = 0x6867666564636261"])
        # test FPU registers
        self.match("register read ft0", ["ft0 = 1.5"])
        self.match("register read ft1", ["ft1 = 3.2053990913985757"])

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("X86")
    def test_x86_64_no_duplicate_subregs(self):
        """Test that duplicate subregisters are not added (on x86_64)."""

        class MyResponder(MockGDBServerResponder):
            reg_data = (
                "0102030405060708"  # rcx
                "1112131415161718"  # rdx
                "2122232425262728"  # rsi
                "3132333435363738"  # rdi
                "4142434445464748"  # rbp
                "5152535455565758"  # rsp
                "6162636465666768"  # r8
                "7172737475767778"  # r9
                "8182838485868788"  # rip
                "91929394"  # eflags
            )

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                        <target>
                          <architecture>i386:x86-64</architecture>
                          <osabi>GNU/Linux</osabi>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="rcx" bitsize="64" type="int64" regnum="2"/>
                            <reg name="rdx" bitsize="64" type="int64" regnum="3"/>
                            <reg name="rsi" bitsize="64" type="int64" regnum="4"/>
                            <reg name="rdi" bitsize="64" type="int64" regnum="5"/>
                            <reg name="rbp" bitsize="64" type="data_ptr" regnum="6"/>
                            <reg name="rsp" bitsize="64" type="data_ptr" regnum="7"/>
                            <reg name="r8" bitsize="64" type="int64" regnum="8"/>
                            <reg name="r9" bitsize="64" type="int64" regnum="9"/>
                            <reg name="rip" bitsize="64" type="code_ptr" regnum="16"/>
                            <reg name="eflags" bitsize="32" type="i386_eflags" regnum="17"/>
                            <reg name="ecx" bitsize="32" type="int" regnum="18" value_regnums="2"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return self.reg_data

            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;07:0102030405060708;10:1112131415161718;"

        self.server.responder = MyResponder()

        target = self.createTarget("basic_eh_frame.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        self.match("register read rcx", ["rcx = 0x0807060504030201"])
        # ecx is supplied via target.xml
        self.match("register read ecx", ["ecx = 0x04030201"])
        self.match("register read rdx", ["rdx = 0x1817161514131211"])
        # edx should not be added
        self.match(
            "register read edx", ["error: Invalid register name 'edx'."], error=True
        )

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("X86")
    def test_i386_no_duplicate_subregs(self):
        """Test that duplicate subregisters are not added (on i386)."""

        class MyResponder(MockGDBServerResponder):
            reg_data = (
                "01020304"  # eax
                "11121314"  # ecx
                "21222324"  # edx
                "31323334"  # ebx
                "41424344"  # esp
                "51525354"  # ebp
                "61626364"  # esi
                "71727374"  # edi
                "81828384"  # eip
                "91929394"  # eflags
            )

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                        <target>
                          <architecture>i386</architecture>
                          <osabi>GNU/Linux</osabi>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="eax" bitsize="32" type="int32" regnum="0"/>
                            <reg name="ecx" bitsize="32" type="int32" regnum="1"/>
                            <reg name="edx" bitsize="32" type="int32" regnum="2"/>
                            <reg name="ebx" bitsize="32" type="int32" regnum="3"/>
                            <reg name="esp" bitsize="32" type="data_ptr" regnum="4"/>
                            <reg name="ebp" bitsize="32" type="data_ptr" regnum="5"/>
                            <reg name="esi" bitsize="32" type="int32" regnum="6"/>
                            <reg name="edi" bitsize="32" type="int32" regnum="7"/>
                            <reg name="eip" bitsize="32" type="code_ptr" regnum="8"/>
                            <reg name="eflags" bitsize="32" type="i386_eflags" regnum="9"/>
                            <reg name="ax" bitsize="16" type="int" regnum="10" value_regnums="0"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return self.reg_data

            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;07:0102030405060708;10:1112131415161718;"

        self.server.responder = MyResponder()

        target = self.createTarget("basic_eh_frame-i386.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        self.match("register read eax", ["eax = 0x04030201"])
        # cx is supplied via target.xml
        self.match("register read ax", ["ax = 0x0201"])
        self.match("register read ecx", ["ecx = 0x14131211"])
        # dx should not be added
        self.match(
            "register read cx", ["error: Invalid register name 'cx'."], error=True
        )

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("AArch64")
    def test_aarch64_no_duplicate_subregs(self):
        """Test that duplicate subregisters are not added."""

        class MyResponder(MockGDBServerResponder):
            reg_data = (
                ("0102030405060708" "1112131415161718")  # x0  # x1
                + 27 * ("2122232425262728")  # x2..x28
                + (
                    "3132333435363738"  # x29 (fp)
                    "4142434445464748"  # x30 (lr)
                    "5152535455565758"  # x31 (sp)
                    "6162636465666768"  # pc
                    "71727374"  # cpsr
                )
            )

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                        <target>
                          <architecture>aarch64</architecture>
                          <feature name="org.gnu.gdb.aarch64.core">
                            <reg name="x0" bitsize="64" type="int" regnum="0"/>
                            <reg name="x1" bitsize="64" type="int" regnum="1"/>
                            <reg name="x2" bitsize="64" type="int" regnum="2"/>
                            <reg name="x3" bitsize="64" type="int" regnum="3"/>
                            <reg name="x4" bitsize="64" type="int" regnum="4"/>
                            <reg name="x5" bitsize="64" type="int" regnum="5"/>
                            <reg name="x6" bitsize="64" type="int" regnum="6"/>
                            <reg name="x7" bitsize="64" type="int" regnum="7"/>
                            <reg name="x8" bitsize="64" type="int" regnum="8"/>
                            <reg name="x9" bitsize="64" type="int" regnum="9"/>
                            <reg name="x10" bitsize="64" type="int" regnum="10"/>
                            <reg name="x11" bitsize="64" type="int" regnum="11"/>
                            <reg name="x12" bitsize="64" type="int" regnum="12"/>
                            <reg name="x13" bitsize="64" type="int" regnum="13"/>
                            <reg name="x14" bitsize="64" type="int" regnum="14"/>
                            <reg name="x15" bitsize="64" type="int" regnum="15"/>
                            <reg name="x16" bitsize="64" type="int" regnum="16"/>
                            <reg name="x17" bitsize="64" type="int" regnum="17"/>
                            <reg name="x18" bitsize="64" type="int" regnum="18"/>
                            <reg name="x19" bitsize="64" type="int" regnum="19"/>
                            <reg name="x20" bitsize="64" type="int" regnum="20"/>
                            <reg name="x21" bitsize="64" type="int" regnum="21"/>
                            <reg name="x22" bitsize="64" type="int" regnum="22"/>
                            <reg name="x23" bitsize="64" type="int" regnum="23"/>
                            <reg name="x24" bitsize="64" type="int" regnum="24"/>
                            <reg name="x25" bitsize="64" type="int" regnum="25"/>
                            <reg name="x26" bitsize="64" type="int" regnum="26"/>
                            <reg name="x27" bitsize="64" type="int" regnum="27"/>
                            <reg name="x28" bitsize="64" type="int" regnum="28"/>
                            <reg name="x29" bitsize="64" type="int" regnum="29"/>
                            <reg name="x30" bitsize="64" type="int" regnum="30"/>
                            <reg name="sp" bitsize="64" type="data_ptr" regnum="31"/>
                            <reg name="pc" bitsize="64" type="code_ptr" regnum="32"/>
                            <reg name="cpsr" bitsize="32" type="cpsr_flags" regnum="33"/>
                            <reg name="w0" bitsize="32" type="int" regnum="34" value_regnums="0"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return self.reg_data

            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;07:0102030405060708;10:1112131415161718;"

        self.server.responder = MyResponder()

        target = self.createTarget("basic_eh_frame-aarch64.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        self.match("register read x0", ["x0 = 0x0807060504030201"])
        # w0 comes from target.xml
        self.match("register read w0", ["w0 = 0x04030201"])
        self.match("register read x1", ["x1 = 0x1817161514131211"])
        # w1 should not be added
        self.match(
            "register read w1", ["error: Invalid register name 'w1'."], error=True
        )



"""
Test symbol table access for main.m.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FoundationSymtabTestCase(TestBase):
    symbols_list = [
        "-[MyString initWithNSString:]",
        "-[MyString dealloc]",
        "-[MyString description]",
        "-[MyString descriptionPauses]",  # synthesized property
        "-[MyString setDescriptionPauses:]",  # synthesized property
        "Test_Selector",
        "Test_NSString",
        "Test_MyString",
        "Test_NSArray",
        "main",
    ]

    @add_test_categories(["pyapi"])
    def test_with_python_api(self):
        """Test symbol table access with Python APIs."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Create the filespec by which to locate our a.out module. Use the
        # absolute path to get the module for the current variant.
        filespec = lldb.SBFileSpec(exe, False)

        module = target.FindModule(filespec)
        self.assertTrue(module, VALID_MODULE)

        # Create the set of known symbols.  As we iterate through the symbol
        # table, remove the symbol from the set if it is a known symbol.
        expected_symbols = set(self.symbols_list)
        for symbol in module:
            self.assertTrue(symbol, VALID_SYMBOL)
            self.trace("symbol:", symbol)
            name = symbol.GetName()
            if name in expected_symbols:
                self.trace(
                    "Removing %s from known_symbols %s" % (name, expected_symbols)
                )
                expected_symbols.remove(name)

        # At this point, the known_symbols set should have become an empty set.
        # If not, raise an error.
        self.trace("symbols unaccounted for:", expected_symbols)
        self.assertEqual(
            len(expected_symbols), 0, "All the known symbols are accounted for"
        )



"""Test Python APIs for working with formatters"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SBFormattersAPITestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_formatters_api(self):
        """Test Python APIs for working with formatters"""
        self.build()
        self.setTearDownCleanup()

        """Test Python APIs for working with formatters"""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synthetic clear", check=False)
            self.runCmd("type category delete foobar", check=False)
            self.runCmd("type category delete JASSynth", check=False)
            self.runCmd("type category delete newbar", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        format = lldb.SBTypeFormat(lldb.eFormatHex)
        category = self.dbg.GetDefaultCategory()
        category.AddTypeFormat(lldb.SBTypeNameSpecifier("int"), format)

        self.expect("frame variable foo.A", substrs=["0x00000001"])
        self.expect("frame variable foo.E", matching=False, substrs=["b8cca70a"])

        category.AddTypeFormat(lldb.SBTypeNameSpecifier("long"), format)
        self.expect("frame variable foo.A", substrs=["0x00000001"])
        self.expect("frame variable foo.E", substrs=["b8cca70a"])

        format.SetFormat(lldb.eFormatOctal)
        category.AddTypeFormat(lldb.SBTypeNameSpecifier("int"), format)
        self.expect("frame variable foo.A", substrs=[" 01"])
        self.expect("frame variable foo.E", substrs=["b8cca70a"])

        category.DeleteTypeFormat(lldb.SBTypeNameSpecifier("int"))
        category.DeleteTypeFormat(lldb.SBTypeNameSpecifier("long"))
        self.expect("frame variable foo.A", matching=False, substrs=[" 01"])
        self.expect("frame variable foo.E", matching=False, substrs=["b8cca70a"])

        summary = lldb.SBTypeSummary.CreateWithSummaryString(
            "the hello world you'll never see"
        )
        summary.SetSummaryString("hello world")
        new_category = self.dbg.GetCategory("foobar")
        self.assertFalse(
            new_category.IsValid(), "getting a non-existing category worked"
        )
        new_category = self.dbg.CreateCategory("foobar")
        new_category.SetEnabled(True)
        new_category.AddTypeSummary(
            lldb.SBTypeNameSpecifier(
                "^.*t$",
                True,  # is_regexp
            ),
            summary,
        )

        self.expect("frame variable foo.A", substrs=["hello world"])
        self.expect("frame variable foo.E", matching=False, substrs=["hello world"])
        self.expect("frame variable foo.B", substrs=["hello world"])
        self.expect("frame variable foo.F", substrs=["hello world"])
        new_category.SetEnabled(False)
        self.expect("frame variable foo.A", matching=False, substrs=["hello world"])
        self.expect("frame variable foo.E", matching=False, substrs=["hello world"])
        self.expect("frame variable foo.B", matching=False, substrs=["hello world"])
        self.expect("frame variable foo.F", matching=False, substrs=["hello world"])
        self.dbg.DeleteCategory(new_category.GetName())
        self.expect("frame variable foo.A", matching=False, substrs=["hello world"])
        self.expect("frame variable foo.E", matching=False, substrs=["hello world"])
        self.expect("frame variable foo.B", matching=False, substrs=["hello world"])
        self.expect("frame variable foo.F", matching=False, substrs=["hello world"])

        filter = lldb.SBTypeFilter(0)
        filter.AppendExpressionPath("A")
        filter.AppendExpressionPath("D")
        self.assertEqual(
            filter.GetNumberOfExpressionPaths(),
            2,
            "filter with two items does not have two items",
        )

        category.AddTypeFilter(lldb.SBTypeNameSpecifier("JustAStruct"), filter)
        self.expect("frame variable foo", substrs=["A = 1", "D = 6.28"])
        self.expect(
            "frame variable foo",
            matching=False,
            substrs=["B = ", "C = ", "E = ", "F = "],
        )

        category.DeleteTypeFilter(lldb.SBTypeNameSpecifier("JustAStruct", True))
        self.expect("frame variable foo", substrs=["A = 1", "D = 6.28"])
        self.expect(
            "frame variable foo",
            matching=False,
            substrs=["B = ", "C = ", "E = ", "F = "],
        )

        category.DeleteTypeFilter(lldb.SBTypeNameSpecifier("JustAStruct", False))
        self.expect("frame variable foo", substrs=["A = 1", "D = 6.28"])
        self.expect(
            "frame variable foo",
            matching=True,
            substrs=["B = ", "C = ", "E = ", "F = "],
        )

        self.runCmd("command script import --allow-reload ./synth.py")

        self.expect("frame variable foo", matching=False, substrs=["X = 1"])

        self.dbg.GetCategory("JASSynth").SetEnabled(True)
        self.expect("frame variable foo", matching=True, substrs=["X = 1"])

        self.dbg.GetCategory("CCCSynth2").SetEnabled(True)
        self.expect(
            "frame variable ccc",
            matching=True,
            substrs=[
                "CCC object with leading synthetic value (int) b = 222",
                "a = 111",
                "b = 222",
                "c = 333",
            ],
        )
        self.dbg.GetCategory("CCCSynth2").SetEnabled(False)

        self.dbg.GetCategory("CCCSynth").SetEnabled(True)
        self.expect(
            "frame variable ccc",
            matching=True,
            substrs=[
                "CCC object with leading value (int) a = 111",
                "a = 111",
                "b = 222",
                "c = 333",
            ],
        )

        self.dbg.GetCategory("BarIntSynth").SetEnabled(True)
        self.expect(
            "frame variable bar_int",
            matching=True,
            substrs=[
                "(int) bar_int = 20 bar_int synthetic: No value",
            ],
        )

        foo_var = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
            .FindVariable("foo")
        )
        self.assertTrue(foo_var.IsValid(), "could not find foo")
        self.assertTrue(
            foo_var.GetDeclaration().IsValid(), "foo declaration is invalid"
        )

        self.assertEqual(
            foo_var.GetNumChildren(),
            2,
            "synthetic value has wrong number of child items (synth)",
        )
        self.assertEqual(
            foo_var.GetChildMemberWithName("X").GetValueAsUnsigned(),
            1,
            "foo_synth.X has wrong value (synth)",
        )
        self.assertFalse(
            foo_var.GetChildMemberWithName("B").IsValid(),
            "foo_synth.B is valid but should not (synth)",
        )

        self.dbg.GetCategory("JASSynth").SetEnabled(False)
        foo_var = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
            .FindVariable("foo")
        )
        self.assertTrue(foo_var.IsValid(), "could not find foo")

        self.assertNotEqual(foo_var.GetNumChildren(), 2, "still seeing synthetic value")

        filter = lldb.SBTypeFilter(0)
        filter.AppendExpressionPath("A")
        filter.AppendExpressionPath("D")
        category.AddTypeFilter(lldb.SBTypeNameSpecifier("JustAStruct"), filter)
        self.expect("frame variable foo", substrs=["A = 1", "D = 6.28"])

        foo_var = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
            .FindVariable("foo")
        )
        self.assertTrue(foo_var.IsValid(), "could not find foo")

        self.assertEqual(
            foo_var.GetNumChildren(),
            2,
            "synthetic value has wrong number of child items (filter)",
        )
        self.assertEqual(
            foo_var.GetChildMemberWithName("X").GetValueAsUnsigned(),
            0,
            "foo_synth.X has wrong value (filter)",
        )
        self.assertEqual(
            foo_var.GetChildMemberWithName("A").GetValueAsUnsigned(),
            1,
            "foo_synth.A has wrong value (filter)",
        )

        self.assertTrue(
            filter.ReplaceExpressionPathAtIndex(0, "C"),
            "failed to replace an expression path in filter",
        )
        self.expect("frame variable foo", substrs=["A = 1", "D = 6.28"])
        category.AddTypeFilter(lldb.SBTypeNameSpecifier("JustAStruct"), filter)
        self.expect("frame variable foo", substrs=["C = 'e'", "D = 6.28"])
        category.AddTypeFilter(lldb.SBTypeNameSpecifier("FooType"), filter)
        filter.ReplaceExpressionPathAtIndex(1, "F")
        self.expect("frame variable foo", substrs=["C = 'e'", "D = 6.28"])
        category.AddTypeFilter(lldb.SBTypeNameSpecifier("JustAStruct"), filter)
        self.expect("frame variable foo", substrs=["C = 'e'", "F = 0"])
        self.expect("frame variable bar", substrs=["C = 'e'", "D = 6.28"])

        foo_var = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
            .FindVariable("foo")
        )
        self.assertTrue(foo_var.IsValid(), "could not find foo")
        self.assertEqual(
            foo_var.GetChildMemberWithName("C").GetValueAsUnsigned(),
            ord("e"),
            "foo_synth.C has wrong value (filter)",
        )

        chosen = self.dbg.GetFilterForType(lldb.SBTypeNameSpecifier("JustAStruct"))
        self.assertEqual(chosen.count, 2, "wrong filter found for JustAStruct")
        self.assertEqual(
            chosen.GetExpressionPathAtIndex(0),
            "C",
            "wrong item at index 0 for JustAStruct",
        )
        self.assertEqual(
            chosen.GetExpressionPathAtIndex(1),
            "F",
            "wrong item at index 1 for JustAStruct",
        )

        self.assertFalse(
            category.DeleteTypeFilter(lldb.SBTypeNameSpecifier("NoSuchType")),
            "deleting a non-existing filter worked",
        )
        self.assertFalse(
            category.DeleteTypeSummary(lldb.SBTypeNameSpecifier("NoSuchType")),
            "deleting a non-existing summary worked",
        )
        self.assertFalse(
            category.DeleteTypeFormat(lldb.SBTypeNameSpecifier("NoSuchType")),
            "deleting a non-existing format worked",
        )
        self.assertFalse(
            category.DeleteTypeSynthetic(lldb.SBTypeNameSpecifier("NoSuchType")),
            "deleting a non-existing synthetic worked",
        )

        self.assertFalse(
            category.DeleteTypeFilter(lldb.SBTypeNameSpecifier("")),
            "deleting a filter for '' worked",
        )
        self.assertFalse(
            category.DeleteTypeSummary(lldb.SBTypeNameSpecifier("")),
            "deleting a summary for '' worked",
        )
        self.assertFalse(
            category.DeleteTypeFormat(lldb.SBTypeNameSpecifier("")),
            "deleting a format for '' worked",
        )
        self.assertFalse(
            category.DeleteTypeSynthetic(lldb.SBTypeNameSpecifier("")),
            "deleting a synthetic for '' worked",
        )

        try:
            self.assertFalse(
                category.AddTypeSummary(lldb.SBTypeNameSpecifier("NoneSuchType"), None),
                "adding a summary valued None worked",
            )
        except:
            pass
        else:
            self.assertFalse(True, "adding a summary valued None worked")

        try:
            self.assertFalse(
                category.AddTypeFilter(lldb.SBTypeNameSpecifier("NoneSuchType"), None),
                "adding a filter valued None worked",
            )
        except:
            pass
        else:
            self.assertFalse(True, "adding a filter valued None worked")

        try:
            self.assertFalse(
                category.AddTypeSynthetic(
                    lldb.SBTypeNameSpecifier("NoneSuchType"), None
                ),
                "adding a synthetic valued None worked",
            )
        except:
            pass
        else:
            self.assertFalse(True, "adding a synthetic valued None worked")

        try:
            self.assertFalse(
                category.AddTypeFormat(lldb.SBTypeNameSpecifier("NoneSuchType"), None),
                "adding a format valued None worked",
            )
        except:
            pass
        else:
            self.assertFalse(True, "adding a format valued None worked")

        self.assertFalse(
            category.AddTypeSummary(
                lldb.SBTypeNameSpecifier("EmptySuchType"), lldb.SBTypeSummary()
            ),
            "adding a summary without value worked",
        )
        self.assertFalse(
            category.AddTypeFilter(
                lldb.SBTypeNameSpecifier("EmptySuchType"), lldb.SBTypeFilter()
            ),
            "adding a filter without value worked",
        )
        self.assertFalse(
            category.AddTypeSynthetic(
                lldb.SBTypeNameSpecifier("EmptySuchType"), lldb.SBTypeSynthetic()
            ),
            "adding a synthetic without value worked",
        )
        self.assertFalse(
            category.AddTypeFormat(
                lldb.SBTypeNameSpecifier("EmptySuchType"), lldb.SBTypeFormat()
            ),
            "adding a format without value worked",
        )

        self.assertFalse(
            category.AddTypeSummary(
                lldb.SBTypeNameSpecifier(""),
                lldb.SBTypeSummary.CreateWithSummaryString(""),
            ),
            "adding a summary for an invalid type worked",
        )
        self.assertFalse(
            category.AddTypeFilter(lldb.SBTypeNameSpecifier(""), lldb.SBTypeFilter(0)),
            "adding a filter for an invalid type worked",
        )
        self.assertFalse(
            category.AddTypeSynthetic(
                lldb.SBTypeNameSpecifier(""),
                lldb.SBTypeSynthetic.CreateWithClassName(""),
            ),
            "adding a synthetic for an invalid type worked",
        )
        self.assertFalse(
            category.AddTypeFormat(
                lldb.SBTypeNameSpecifier(""), lldb.SBTypeFormat(lldb.eFormatHex)
            ),
            "adding a format for an invalid type worked",
        )

        new_category = self.dbg.CreateCategory("newbar")
        new_category.AddTypeSummary(
            lldb.SBTypeNameSpecifier("JustAStruct"),
            lldb.SBTypeSummary.CreateWithScriptCode("return 'hello scripted world';"),
        )
        self.expect(
            "frame variable foo", matching=False, substrs=["hello scripted world"]
        )
        new_category.SetEnabled(True)
        self.expect(
            "frame variable foo", matching=True, substrs=["hello scripted world"]
        )

        self.expect(
            "frame variable foo_ptr", matching=True, substrs=["hello scripted world"]
        )
        new_category.AddTypeSummary(
            lldb.SBTypeNameSpecifier("JustAStruct"),
            lldb.SBTypeSummary.CreateWithScriptCode(
                "return 'hello scripted world';", lldb.eTypeOptionSkipPointers
            ),
        )
        self.expect(
            "frame variable foo", matching=True, substrs=["hello scripted world"]
        )

        frame = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
        )
        foo_ptr = frame.FindVariable("foo_ptr")
        summary = foo_ptr.GetTypeSummary()

        self.assertFalse(
            summary.IsValid(), "summary found for foo* when none was planned"
        )

        self.expect(
            "frame variable foo_ptr", matching=False, substrs=["hello scripted world"]
        )

        new_category.AddTypeSummary(
            lldb.SBTypeNameSpecifier("JustAStruct"),
            lldb.SBTypeSummary.CreateWithSummaryString(
                "hello static world", lldb.eTypeOptionNone
            ),
        )

        summary = foo_ptr.GetTypeSummary()

        self.assertTrue(
            summary.IsValid(), "no summary found for foo* when one was in place"
        )
        self.assertEqual(
            summary.GetData(), "hello static world", "wrong summary found for foo*"
        )

        self.expect("frame variable e1", substrs=["I am an empty Empty1 {}"])
        self.expect("frame variable e2", substrs=["I am an empty Empty2"])
        self.expect(
            "frame variable e2", substrs=["I am an empty Empty2 {}"], matching=False
        )

        self.assertIsNotNone(
            self.dbg.GetCategory(lldb.eLanguageTypeObjC), "ObjC category is None"
        )

    def test_force_synth_off(self):
        """Test that one can have the public API return non-synthetic SBValues if desired"""
        self.build(dictionary={"EXE": "no_synth"})
        self.setTearDownCleanup()

        self.runCmd("file " + self.getBuildArtifact("no_synth"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synthetic clear", check=False)
            self.runCmd("type category delete foobar", check=False)
            self.runCmd("type category delete JASSynth", check=False)
            self.runCmd("type category delete newbar", check=False)
            self.runCmd("settings set target.enable-synthetic-value true")

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        frame = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
        )
        int_vector = frame.FindVariable("int_vector")
        if self.TraceOn():
            print(int_vector)
        self.assertEqual(int_vector.GetNumChildren(), 0, "synthetic vector is empty")

        self.runCmd("settings set target.enable-synthetic-value false")
        frame = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
        )
        int_vector = frame.FindVariable("int_vector")
        if self.TraceOn():
            print(int_vector)
        self.assertNotEqual(
            int_vector.GetNumChildren(), 0, '"physical" vector is not empty'
        )

        self.runCmd("settings set target.enable-synthetic-value true")
        frame = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
        )
        int_vector = frame.FindVariable("int_vector")
        if self.TraceOn():
            print(int_vector)
        self.assertEqual(
            int_vector.GetNumChildren(), 0, "synthetic vector is still empty"
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestThreadSelectionBug(GDBRemoteTestBase):
    def test(self):
        class MyResponder(MockGDBServerResponder):
            def cont(self):
                # Simulate process stopping due to a raise(SIGINT)
                return "T01reason:signal"

        self.server.responder = MyResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        python_os_plugin_path = os.path.join(self.getSourceDir(), "operating_system.py")
        command = "settings set target.process.python-os-plugin-path '{}'".format(
            python_os_plugin_path
        )
        self.dbg.HandleCommand(command)

        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetNumThreads(), 3)

        # Verify our OS plug-in threads showed up
        thread = process.GetThreadByID(0x1)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x1 after we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x2)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x2 after we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x3)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x3 after we load the python OS plug-in",
        )

        # Verify that a thread other than 3 is selected.
        thread = process.GetSelectedThread()
        self.assertNotEqual(thread.GetThreadID(), 0x3)

        # Verify that we select the thread backed by physical thread 1, rather
        # than virtual thread 1. The mapping comes from the OS plugin, where we
        # specified that thread 3 is backed by real thread 1.
        process.Continue()
        thread = process.GetSelectedThread()
        self.assertEqual(thread.GetThreadID(), 0x3)



"""
Tests the builtin formats of LLDB.
"""

import lldb
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestCase(TestBase):
    def getFormatted(self, format, expr):
        """
        Evaluates the expression and formats the result with the given format.
        """
        result = lldb.SBCommandReturnObject()
        full_expr = "expr --format '" + format + "' -- " + expr
        self.dbg.GetCommandInterpreter().HandleCommand(full_expr, result)
        self.assertTrue(result.Succeeded(), result.GetError())
        return result.GetOutput()

    @skipIf(dwarf_version=["<", "3"])
    @no_debug_info_test
    @skipIfWindows
    def testAllPlatforms(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        # We can dump correctly non char* c-strings with explicit formatting.
        self.assertIn(' = ""', self.getFormatted("c-string", "void_empty_cstring"))
        self.assertIn(' = ""', self.getFormatted("c-string", "empty_cstring"))

    # TODO: Move as many asserts as possible within this function to `testAllPlatforms`.
    # Currently `arm` is being skipped even though many asserts would effectively
    # pass.
    @no_debug_info_test
    @skipIfWindows
    # uint128_t not available on arm.
    @skipIf(archs=["arm$"])
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # void
        self.assertEqual("", self.getFormatted("void", "1"))

        # boolean
        self.assertIn("= false\n", self.getFormatted("boolean", "0"))
        self.assertIn("= true\n", self.getFormatted("boolean", "1"))
        self.assertIn("= true\n", self.getFormatted("boolean", "2"))
        self.assertIn(
            "= error: unsupported byte size (16) for boolean format\n",
            self.getFormatted("boolean", "(__uint128_t)0"),
        )

        # float
        self.assertIn("= 0\n", self.getFormatted("float", "0"))
        self.assertIn("= 2\n", self.getFormatted("float", "0x40000000"))
        self.assertIn("= NaN\n", self.getFormatted("float", "-1"))
        # Checks the float16 code.
        self.assertIn("= 2\n", self.getFormatted("float", "(__UINT16_TYPE__)0x4000"))
        self.assertIn(
            "= error: unsupported byte size (1) for float format\n",
            self.getFormatted("float", "'a'"),
        )

        # enumeration
        self.assertIn("= 0\n", self.getFormatted("enumeration", "0"))
        self.assertIn("= 1234567\n", self.getFormatted("enumeration", "1234567"))
        self.assertIn("= -1234567\n", self.getFormatted("enumeration", "-1234567"))

        # dec
        self.assertIn("= 1234567\n", self.getFormatted("dec", "1234567"))
        self.assertIn(
            "= 123456789\n", self.getFormatted("dec", "(__uint128_t)123456789")
        )

        # unsigned decimal
        self.assertIn("= 1234567\n", self.getFormatted("unsigned decimal", "1234567"))
        self.assertIn(
            "= 4293732729\n", self.getFormatted("unsigned decimal", "-1234567")
        )
        self.assertIn(
            "= 123456789\n",
            self.getFormatted("unsigned decimal", "(__uint128_t)123456789"),
        )

        # octal
        self.assertIn("= 04553207\n", self.getFormatted("octal", "1234567"))
        self.assertIn(
            "= 0221505317046536757\n",
            self.getFormatted("octal", "(__uint128_t)0x123456789ABDEFull"),
        )

        # complex float
        self.assertIn(
            "= error: unsupported byte size (1) for complex float format\n",
            self.getFormatted("complex float", "'a'"),
        )

        # complex integer
        self.assertIn(
            "= error: unsupported byte size (1) for complex integer format\n",
            self.getFormatted("complex integer", "'a'"),
        )

        # hex
        self.assertIn("= 0x00abc123\n", self.getFormatted("hex", "0xABC123"))
        self.assertIn(
            "= 0x000000000000000000123456789abdef\n",
            self.getFormatted("hex", "(__uint128_t)0x123456789ABDEFull"),
        )

        # hex float
        self.assertIn("= 0x1p1\n", self.getFormatted("hex float", "2.0f"))
        self.assertIn("= 0x1p1\n", self.getFormatted("hex float", "2.0"))
        # FIXME: long double not supported.
        # on Darwin arm64, long double is 8 bytes, same as long.
        if self.getArchitecture() != "arm64":
            self.assertIn(
                "= error: unsupported byte size (16) for hex float format\n",
                self.getFormatted("hex float", "2.0l"),
            )

        # uppercase hex
        self.assertIn("= 0x00ABC123\n", self.getFormatted("uppercase hex", "0xABC123"))

        # binary
        self.assertIn(
            "= 0b00000000000000000000000000000010\n", self.getFormatted("binary", "2")
        )
        self.assertIn("= 0b01100001\n", self.getFormatted("binary", "'a'"))
        self.assertIn(
            " = 0b10010001101000101011001111000\n",
            self.getFormatted("binary", "(__uint128_t)0x12345678ll"),
        )

        # Different character arrays.
        # FIXME: Passing a 'const char *' will ignore any given format,
        self.assertIn(
            r'= " \U0000001b\a\b\f\n\r\t\vaA09"',
            self.getFormatted("character array", "cstring"),
        )
        self.assertIn(
            r'= " \U0000001b\a\b\f\n\r\t\vaA09"',
            self.getFormatted("c-string", "cstring"),
        )
        self.assertIn(
            ' = " \\e\\a\\b\\f\\n\\r\\t\\vaA09" " \\U0000001b\\a\\b\\f\\n\\r\\t\\vaA09"\n',
            self.getFormatted("c-string", "(char *)cstring"),
        )
        self.assertIn("=\n", self.getFormatted("c-string", "(__UINT64_TYPE__)0"))

        # Build a uint128_t that contains a series of characters in each byte.
        # First 8 byte of the uint128_t.
        cstring_chars1 = " \a\b\f\n\r\t\v"
        # Last 8 byte of the uint128_t.
        cstring_chars2 = "AZaz09\033\0"

        # Build a uint128_t value with the hex encoded characters.
        string_expr = "((__uint128_t)0x"
        for c in cstring_chars1:
            string_expr += format(ord(c), "x").zfill(2)
        string_expr += "ull << 64) | (__uint128_t)0x"
        for c in cstring_chars2:
            string_expr += format(ord(c), "x").zfill(2)
        string_expr += "ull"

        # Try to print that uint128_t with the different char formatters.
        self.assertIn(
            "= \\0\\e90zaZA\\v\\t\\r\\n\\f\\b\\a \n",
            self.getFormatted("character array", string_expr),
        )
        self.assertIn(
            "= \\0\\e90zaZA\\v\\t\\r\\n\\f\\b\\a \n",
            self.getFormatted("character", string_expr),
        )
        self.assertIn(
            "= ..90zaZA....... \n",
            self.getFormatted("printable character", string_expr),
        )
        self.assertIn(
            "= 0x00 0x1b 0x39 0x30 0x7a 0x61 0x5a 0x41 0x0b 0x09 0x0d 0x0a 0x0c 0x08 0x07 0x20\n",
            self.getFormatted("unicode8", string_expr),
        )

        # OSType
        ostype_expr = "(__UINT64_TYPE__)0x"
        for c in cstring_chars1:
            ostype_expr += format(ord(c), "x").zfill(2)
        self.assertIn(
            "= ' \\a\\b\\f\\n\\r\\t\\v'\n", self.getFormatted("OSType", ostype_expr)
        )

        ostype_expr = "(__UINT64_TYPE__)0x"
        for c in cstring_chars2:
            ostype_expr += format(ord(c), "x").zfill(2)
        self.assertIn("= 'AZaz09\\e\\0'\n", self.getFormatted("OSType", ostype_expr))

        self.assertIn(
            "= 0x2007080c0a0d090b415a617a30391b00\n",
            self.getFormatted("OSType", string_expr),
        )

        # bytes
        self.assertIn(
            r'= " \U0000001b\a\b\f\n\r\t\vaA09"', self.getFormatted("bytes", "cstring")
        )

        # bytes with ASCII
        self.assertIn(
            r'= " \U0000001b\a\b\f\n\r\t\vaA09"',
            self.getFormatted("bytes with ASCII", "cstring"),
        )

        # unicode8
        self.assertIn(
            "= 0x78 0x56 0x34 0x12\n", self.getFormatted("unicode8", "0x12345678")
        )

        # unicode16
        self.assertIn("= U+5678 U+1234\n", self.getFormatted("unicode16", "0x12345678"))

        # unicode32
        self.assertIn(
            "= U+0x89abcdef U+0x01234567\n",
            self.getFormatted("unicode32", "(__UINT64_TYPE__)0x123456789ABCDEFll"),
        )

        # address
        self.assertIn("= 0x00000012\n", self.getFormatted("address", "0x12"))
        self.assertIn("= 0x00000000\n", self.getFormatted("address", "0"))

        # Different fixed-width integer type arrays (e.g. 'uint8_t[]').
        self.assertIn(
            "= {0xf8 0x56 0x34 0x12}\n", self.getFormatted("uint8_t[]", "0x123456f8")
        )
        self.assertIn("= {-8 86 52 18}\n", self.getFormatted("int8_t[]", "0x123456f8"))

        self.assertIn(
            "= {0x56f8 0x1234}\n", self.getFormatted("uint16_t[]", "0x123456f8")
        )
        self.assertIn("= {-2312 4660}\n", self.getFormatted("int16_t[]", "0x1234F6f8"))

        self.assertIn(
            "= {0x89abcdef 0x01234567}\n",
            self.getFormatted("uint32_t[]", "(__UINT64_TYPE__)0x123456789ABCDEFll"),
        )
        self.assertIn(
            "= {-1985229329 19088743}\n",
            self.getFormatted("int32_t[]", "(__UINT64_TYPE__)0x123456789ABCDEFll"),
        )

        self.assertIn(
            "= {0x89abcdef 0x01234567 0x00000000 0x00000000}\n",
            self.getFormatted("uint32_t[]", "__uint128_t i = 0x123456789ABCDEF; i"),
        )
        self.assertIn(
            "= {-1985229329 19088743 0 0}\n",
            self.getFormatted("int32_t[]", "__uint128_t i = 0x123456789ABCDEF; i"),
        )

        self.assertIn(
            "= {0x0123456789abcdef 0x0000000000000000}\n",
            self.getFormatted("uint64_t[]", "__uint128_t i = 0x123456789ABCDEF; i"),
        )
        self.assertIn(
            "= {-994074541749903617 0}\n",
            self.getFormatted("int64_t[]", "__uint128_t i = 0xF23456789ABCDEFFll; i"),
        )

        # There is not int128_t[] style, so this only tests uint128_t[].
        self.assertIn(
            "= {0x00000000000000000123456789abcdef}\n",
            self.getFormatted("uint128_t[]", "__uint128_t i = 0x123456789ABCDEF; i"),
        )

        # Different fixed-width float type arrays.
        self.assertIn("{2 2}\n", self.getFormatted("float16[]", "0x40004000"))
        self.assertIn("{2 2}\n", self.getFormatted("float32[]", "0x4000000040000000ll"))
        self.assertIn(
            "{2 0}\n",
            self.getFormatted("float64[]", "__uint128_t i = 0x4000000000000000ll; i"),
        )

        # Invalid format string
        self.expect(
            "expr --format invalid_format_string -- 1",
            error=True,
            substrs=[
                "error: Invalid format character or name 'invalid_format_string'. Valid values are:"
            ],
        )

    # Extends to host target pointer width.
    @skipIf(archs=no_match(["x86_64"]))
    @no_debug_info_test
    def test_pointer(self):
        # pointer
        self.assertIn("= 0x000000000012d687\n", self.getFormatted("pointer", "1234567"))
        self.assertIn("= 0x0000000000000000\n", self.getFormatted("pointer", "0"))
        # FIXME: Just ignores the input value as it's not pointer sized.
        self.assertIn("= 0x0000000000000000\n", self.getFormatted("pointer", "'a'"))

    # Depends on the host target for decoding.
    @skipIf(archs=no_match(["x86_64"]))
    @no_debug_info_test
    def test_instruction(self):
        self.assertIn(
            "= addq   0xa(%rdi), %r8\n", self.getFormatted("instruction", "0x0a47034c")
        )
